Method: Immutable::Hash#>
- Defined in:
- lib/immutable/hash.rb
#>(other) ⇒ Boolean
Return true if this Hash is a proper superset of other, which means all other‘s keys are contained in this Hash with identical values, and the two hashes are not identical.
783 784 785 |
# File 'lib/immutable/hash.rb', line 783 def >(other) self != other && self >= other end |