Method: Immutable::Vector#flat_map
- Defined in:
- lib/immutable/vector.rb
#flat_map ⇒ Vector
Return a new Vector with the concatenated results of running the block once for every element in this Vector.
513 514 515 516 517 |
# File 'lib/immutable/vector.rb', line 513 def flat_map return enum_for(:flat_map) if not block_given? return self if empty? self.class.new(super) end |